﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Data.MoveVectorData;
using HeuristicLab.Data.MoveVectorData.Interfaces;
using HeuristicLab.Data.MoveVectorData.Moves;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.Encodings.MoveVectorEncoding.Manipulators
{
    [Item("SingleMoveChangeManipulator", " Uniformly distributed change of a single position of a move.")]
    [StorableClass]
    public class SingleMoveChangeManipulator : BoundedMoveVectorManipulator
    {
        [StorableConstructor]
        protected SingleMoveChangeManipulator(bool deserializing) : base(deserializing) { }
        protected SingleMoveChangeManipulator(SingleMoveChangeManipulator original, Cloner cloner) : base(original, cloner) { }

        public SingleMoveChangeManipulator() : base() { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new SingleMoveChangeManipulator(this, cloner);
        }

        public static void Apply(IRandom random, MoveVector moves, IntMatrix bounds)
        {
            Manipulate(random, moves, bounds, random.Next(moves.Length));
        }

        public static void Manipulate(IRandom random, MoveVector moves, IntMatrix bounds, int index)
        {
            if (bounds == null || bounds.Rows == 0 || bounds.Columns < 2) throw new ArgumentException("SingleMoveChangeManipulator: Invalid bounds specified", "bounds");
            int min = bounds[index % bounds.Rows, 0], max = bounds[index % bounds.Rows, 1], step = 1;
            if (bounds.Columns > 2) step = bounds[index % bounds.Rows, 2];

            Type type = moves.MoveTypes[random.Next(0, moves.MoveTypes.Count)];
            IMove move = (IMove)Activator.CreateInstance(type);
            move.Randomize(random, min, max, step);
            moves[index] = move;
        }

        protected override void ManipulateBounded(IRandom random, MoveVector moves, IntMatrix bounds)
        {
            if (BoundsParameter.ActualValue == null) throw new InvalidOperationException("SingleMoveChangeManipulator: Parameter " + BoundsParameter.ActualName + " could not be found.");
            Apply(random, moves, bounds);
        }
    }
}
